<?php

namespace Drupal\Tests\openapi_jsonapi\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Drupal\openapi_test\Entity\OpenApiTestEntityType;

/**
 * @coversDefaultClass \Drupal\openapi_jsonapi\Plugin\openapi\OpenApiGenerator\JsonApiGenerator
 *
 * @group openapi_jsonapi
 */
final class JsonApiGeneratorTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'field',
    'link',
    'menu_link_content',
    'filter',
    'text',
    'taxonomy',
    'serialization',
    'jsonapi',
    'openapi',
    'openapi_test',
    'openapi_jsonapi',
    'system',
    'user',
    'menu_ui',
    'file',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->installEntitySchema('user');
    $this->installEntitySchema('taxonomy_term');
    $this->installEntitySchema('menu_link_content');
    $this->installEntitySchema('openapi_test_entity');
    $this->installEntitySchema('file');

    OpenApiTestEntityType::create([
      'id' => 'test',
      'label' => 'Test',
    ])->save();
  }

  /**
   * @covers ::getSpecification
   */
  public function testSpecification() {
    $field_definitions = $this->container
      ->get('entity_field.manager')
      ->getFieldDefinitions('openapi_test_entity', 'test');

    // Assert that the menu_link field is defined on the test entity type.
    // @todo Why is this here?
    $this->assertArrayHasKey('menu_link', $field_definitions);

    $specification = $this->container->get('plugin.manager.openapi.generator')
      ->createInstance('jsonapi')
      ->getSpecification();
    // Very basic validation that we can encode to JSON.
    json_encode($specification, JSON_UNESCAPED_SLASHES|JSON_THROW_ON_ERROR);
    $this->assertEquals('string', $specification['components']['schemas']['openapi_test_entity--test']['properties']['attributes']['properties']['json_data']['properties']['property1']['type']);
  }

}
